module UToy.Parsers
  ( parseHexBytes
  ) where

import Data.Char (isHexDigit, ord)
import Data.Text (Text)
import Data.Word (Word8)
import Text.Printf (printf)

import qualified Data.Attoparsec.Text as Atto

parseHexBytes :: Text -> Either String [Word8]
parseHexBytes = Atto.parseOnly $ hexBytes <* Atto.endOfInput

hexBytes :: Atto.Parser [Word8]
hexBytes = hexByte `Atto.sepBy` separators
  where
    hexByte = do
      high <- hexDigit
      low <- hexDigit
      pure $ fromIntegral $ high * 16 + low

    hexDigit = hexDigitToInt <$> Atto.satisfy isHexDigit

    hexDigitToInt c
      | '0' <= c && c <= '9' = ord c - ord '0'
      | 'A' <= c && c <= 'F' = ord c - ord 'A' + 10
      | 'a' <= c && c <= 'f' = ord c - ord 'a' + 10
      | otherwise = error $ printf "not a hex digit: %c" c
    
    separators = Atto.skipMany $ Atto.satisfy $ Atto.inClass " +."
